@@ -28,8 +28,8 @@ class RExpression : public Cantor::Expression
   Q_OBJECT
   public:
     enum ServerReturnCode{SuccessCode=0, ErrorCode, InterruptedCode};
-    RExpression( Cantor::Session*);
-    ~RExpression();
+    explicit RExpression( Cantor::Session*, bool internal = false);
+    ~RExpression() override = default;
 
     void evaluate() override;
     void interrupt() override;
@@ -38,7 +38,6 @@ class RExpression : public Cantor::Expression
 
   public Q_SLOTS:
     void finished(int returnCode, const QString& text);
-    void evaluationStarted();
     void showFilesAsResult(const QStringList& files);
 
   private:
