@@ -31,20 +31,20 @@
 #include "settings.h"
 #include "juliahighlighter.h"
 #include "juliakeywords.h"
+#include "juliavariablemodel.h"
 #include "juliaextensions.h"
 #include "juliabackend.h"
 #include "juliacompletionobject.h"
-#include <julia/julia_version.h>
+#include <julia_version.h>
 
-const QRegularExpression JuliaSession::typeVariableInfo = QRegularExpression(QLatin1String("\\w+\\["));
+using namespace Cantor;
 
 JuliaSession::JuliaSession(Cantor::Backend *backend)
     : Session(backend)
     , m_process(nullptr)
     , m_interface(nullptr)
-    , m_currentExpression(nullptr)
-    , m_variableModel(new Cantor::DefaultVariableModel(this))
 {
+    setVariableModel(new JuliaVariableModel(this));
 }
 
 void JuliaSession::login()
@@ -101,13 +101,14 @@ void JuliaSession::login()
         JuliaSettings::self()->replPath().path()
     );
 
-    listVariables();
+    static_cast<JuliaVariableModel*>(variableModel())->setJuliaServer(m_interface);
 
     // Plots integration
     if (integratePlots()) {
         runJuliaCommand(
             QLatin1String("import GR; ENV[\"GKS_WSTYPE\"] = \"nul\"")
         );
+        updateVariables();
     }
 
     changeStatus(Session::Done);
@@ -119,25 +120,25 @@ void JuliaSession::logout()
 {
     m_process->terminate();
 
-    JuliaKeywords::instance()->clearVariables();
-    JuliaKeywords::instance()->clearFunctions();
-    m_variableModel->clearVariables();
-    emit updateHighlighter();
+    variableModel()->clearVariables();
 
     changeStatus(Status::Disable);
 }
 
 void JuliaSession::interrupt()
 {
+    if (expressionQueue().isEmpty())
+        return;
+
     if (m_process->pid()) {
         m_process->kill();
     }
 
-    for (Cantor::Expression *e : m_runningExpressions) {
-        e->interrupt();
-    }
+    qDebug()<<"interrupting " << expressionQueue().first()->command();
+    foreach (Cantor::Expression* expression, expressionQueue())
+        expression->setStatus(Cantor::Expression::Interrupted);
+    expressionQueue().clear();
 
-    m_runningExpressions.clear();
     changeStatus(Cantor::Session::Done);
 }
 
@@ -148,8 +149,6 @@ Cantor::Expression *JuliaSession::evaluateExpression(
 {
     JuliaExpression *expr = new JuliaExpression(this, internal);
 
-    changeStatus(Cantor::Session::Running);
-
     expr->setFinishingBehavior(behave);
     expr->setCommand(cmd);
     expr->evaluate();
@@ -166,11 +165,7 @@ Cantor::CompletionObject *JuliaSession::completionFor(
 
 QSyntaxHighlighter *JuliaSession::syntaxHighlighter(QObject *parent)
 {
-    JuliaHighlighter *highlighter = new JuliaHighlighter(parent);
-    QObject::connect(
-        this, SIGNAL(updateHighlighter()), highlighter, SLOT(updateHighlight())
-    );
-    return highlighter;
+    return new JuliaHighlighter(parent, this);
 }
 
 void JuliaSession::runJuliaCommand(const QString &command) const
@@ -190,19 +185,16 @@ void JuliaSession::runJuliaCommandAsync(const QString &command)
 
 void JuliaSession::onResultReady()
 {
-    m_currentExpression->finalize();
-    m_runningExpressions.removeAll(m_currentExpression);
-
-    listVariables();
-
-    changeStatus(Cantor::Session::Done);
+    static_cast<JuliaExpression*>(expressionQueue().first())->finalize(getOutput(), getError(), getWasException());
+    finishFirstExpression(true);
 }
 
-void JuliaSession::runExpression(JuliaExpression *expr)
+void JuliaSession::runFirstExpression()
 {
-    m_runningExpressions.append(expr);
-    m_currentExpression = expr;
-    runJuliaCommandAsync(expr->command());
+    Cantor::Expression* expr = expressionQueue().first();
+    expr->setStatus(Cantor::Expression::Computing);
+
+    runJuliaCommandAsync(expr->internalCommand());
 }
 
 QString JuliaSession::getStringFromServer(const QString &method)
@@ -228,55 +220,6 @@ bool JuliaSession::getWasException()
     return reply.isValid() && reply.value();
 }
 
-void JuliaSession::listVariables()
-{
-    JuliaKeywords::instance()->clearVariables();
-    JuliaKeywords::instance()->clearFunctions();
-
-    m_interface->call(QLatin1String("parseModules"));
-
-    const QStringList& variables =
-        static_cast<QDBusReply<QStringList>>(m_interface->call(QLatin1String("variablesList"))).value();
-    const QStringList& values =
-        static_cast<QDBusReply<QStringList>>(m_interface->call(QLatin1String("variableValuesList"))).value();
-    for (int i = 0; i < variables.size(); i++)
-    {
-        if (i >= values.size())
-        {
-            qWarning() << "Don't have value for variable from julia server response, something wrong!";
-            continue;
-        }
-
-        const QString& name = variables[i];
-        QString value = values[i];
-        if (value != JuliaVariableManagementExtension::REMOVED_VARIABLE_MARKER)
-        {
-            // Register variable
-            // We use replace here, because julia return data type for some variables, and we need
-            // remove it to make variable view more consistent with the other backends
-            // More info: https://bugs.kde.org/show_bug.cgi?id=377771
-            m_variableModel->addVariable(name, value.replace(typeVariableInfo,QLatin1String("[")));
-            JuliaKeywords::instance()->addVariable(name);
-        }
-        else
-            m_variableModel->removeVariable(name);
-    }
-
-    const QStringList& functions =
-        static_cast<QDBusReply<QStringList>>(m_interface->call(QLatin1String("functionsList"))).value();
-    foreach (const QString& name, functions)
-    {
-        JuliaKeywords::instance()->addFunction(name);
-    }
-
-    emit updateHighlighter();
-}
-
-QAbstractItemModel *JuliaSession::variableModel()
-{
-    return m_variableModel;
-}
-
 
 bool JuliaSession::integratePlots()
 {
