@@ -26,6 +26,7 @@
 
 class JuliaExpression;
 class JuliaCompletionObject;
+class JuliaVariableModel;
 class KProcess;
 class QDBusInterface;
 namespace Cantor {
@@ -83,22 +84,11 @@ public:
      */
     QSyntaxHighlighter *syntaxHighlighter(QObject *parent) override;
 
-    /**
-     * @see Cantor::Session::variableModel
-     */
-    QAbstractItemModel *variableModel() override;
-
     /**
      * @return indicator if config says to integrate plots into worksheet
      */
     bool integratePlots();
 
-Q_SIGNALS:
-    /**
-     * Emit this to update syntax highlighter
-     */
-    void updateHighlighter();
-
 private Q_SLOTS:
     /**
      * Called when async call to JuliaServer is finished
@@ -109,26 +99,10 @@ private:
     KProcess *m_process; //< process to run JuliaServer inside
     QDBusInterface *m_interface; //< interface to JuliaServer
 
-    /// Expressions running at the moment
-    QList<JuliaExpression *> m_runningExpressions;
-    JuliaExpression *m_currentExpression; //< current expression
-
-    /// Variable management model
-    Cantor::DefaultVariableModel *m_variableModel;
-    static const QRegularExpression typeVariableInfo;
-
     /// Cache to speedup modules whos calls
     QMap<QString, QString> m_whos_cache;
 
-    friend JuliaExpression;
-    friend JuliaCompletionObject;
-
-    /**
-     * Runs Julia expression
-     *
-     * @param expression expression to run
-     */
-    void runExpression(JuliaExpression *expression);
+    void runFirstExpression() override;
 
     /**
      * Runs Julia piece of code in synchronous mode
@@ -167,9 +141,4 @@ private:
      * @return indicator of exception occurred during the last command execution
      */
     bool getWasException();
-
-    /**
-     * Updates variable model by querying all modules in scope with whos command
-     */
-    void listVariables();
 };
