@@ -49,9 +49,9 @@ const QRegExp MaximaSession::MaximaInputPrompt = QRegExp(QLatin1String("(\\(\\s*
 
 MaximaSession::MaximaSession( Cantor::Backend* backend ) : Session(backend),
     m_process(nullptr),
-    m_variableModel(new MaximaVariableModel(this)),
     m_justRestarted(false)
 {
+    setVariableModel(new MaximaVariableModel(this));
 }
 
 void MaximaSession::login()
@@ -89,6 +89,7 @@ void MaximaSession::login()
         QString autorunScripts = MaximaSettings::self()->autorunScripts().join(QLatin1String(";"));
         autorunScripts.append(QLatin1String(";kill(labels)")); // Reset labels after running autorun scripts
         evaluateExpression(autorunScripts, MaximaExpression::DeleteOnFinish, true);
+        updateVariables();
     }
 
     changeStatus(Session::Done);
@@ -122,7 +123,8 @@ void MaximaSession::logout()
     expressionQueue().clear();
     delete m_process;
     m_process = nullptr;
-    m_variableModel->clear();
+    variableModel()->clearVariables();
+    variableModel()->clearFunctions();
 
     changeStatus(Status::Disable);
 
@@ -193,35 +195,22 @@ void MaximaSession::reportProcessError(QProcess::ProcessError e)
 void MaximaSession::currentExpressionChangedStatus(Cantor::Expression::Status status)
 {
     Cantor::Expression* expression = expressionQueue().first();
+    const QString& cmd = expression->command();
     qDebug() << "expression status changed: command = " << expression->command() << ", status = " << status;
 
-    if(status!=Cantor::Expression::Computing) //The session is ready for the next command
+    switch (status)
     {
+    case Cantor::Expression::Done:
+    case Cantor::Expression::Error:
         qDebug()<<"################################## EXPRESSION END ###############################################";
         disconnect(expression, SIGNAL(statusChanged(Cantor::Expression::Status)),
-                   this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
+            this, SLOT(currentExpressionChangedStatus(Cantor::Expression::Status)));
 
-        expressionQueue().removeFirst();
-        if(expressionQueue().isEmpty())
-        {
-            //if we are done with all the commands in the queue,
-            //use the opportunity to update the variablemodel (if the last command wasn't already an update, as infinite loops aren't fun)
-            QRegExp exp=QRegExp(QRegExp::escape(MaximaVariableModel::inspectCommand).arg(QLatin1String("(values|functions)")));
-            QRegExp exp2=QRegExp(QRegExp::escape(MaximaVariableModel::variableInspectCommand).arg(QLatin1String("(values|functions)")));
+        finishFirstExpression();
+        break;
 
-            if(MaximaSettings::variableManagement()&&!exp.exactMatch(expression->command())&&!exp2.exactMatch(expression->command()))
-            {
-                m_variableModel->checkForNewFunctions();
-                m_variableModel->checkForNewVariables();
-            }else
-            {
-                changeStatus(Cantor::Session::Done);
-            }
-
-        }else
-        {
-            runFirstExpression();
-        }
+    default:
+        break;
     }
 }
 
@@ -265,10 +254,8 @@ void MaximaSession::interrupt()
             ; //TODO: interrupt the process on windows
 #endif
         }
-        expressionQueue().first()->interrupt();
-        expressionQueue().removeFirst();
-        foreach (Cantor::Expression* expression, expressionQueue())
-            expression->setStatus(Cantor::Expression::Done);
+       foreach (Cantor::Expression* expression, expressionQueue())
+            expression->setStatus(Cantor::Expression::Interrupted);
         expressionQueue().clear();
 
         qDebug()<<"done interrupting";
@@ -338,11 +325,6 @@ QSyntaxHighlighter* MaximaSession::syntaxHighlighter(QObject* parent)
     return new MaximaHighlighter(parent, this);
 }
 
-QAbstractItemModel* MaximaSession::variableModel()
-{
-    return m_variableModel;
-}
-
 void MaximaSession::write(const QString& exp) {
     qDebug()<<"################################## EXPRESSION START ###############################################";
     qDebug()<<"sending expression to maxima process: " << exp;
