@@ -40,20 +40,19 @@
 #include <signal.h>
 #endif
 
-#include <defaultvariablemodel.h>
+#include "octavevariablemodel.h"
 
 const QRegExp OctaveSession::PROMPT_UNCHANGEABLE_COMMAND = QRegExp(QLatin1String("(,|;)+"));
 
-OctaveSession::OctaveSession ( Cantor::Backend* backend ) : Session ( backend ),
+OctaveSession::OctaveSession ( Cantor::Backend* backend ) : Session ( backend),
 m_process(nullptr),
 m_prompt(QLatin1String("CANTOR_OCTAVE_BACKEND_PROMPT:([0-9]+)> ")),
 m_subprompt(QLatin1String("CANTOR_OCTAVE_BACKEND_SUBPROMPT:([0-9]+)> ")),
 m_previousPromptNumber(1),
 m_watch(nullptr),
-m_loginFinish(false),
-m_syntaxError(false),
-m_variableModel(new Cantor::DefaultVariableModel(this))
+m_syntaxError(false)
 {
+    setVariableModel(new OctaveVariableModel(this));
     qDebug() << octaveScriptInstallDir;
 }
 
@@ -78,13 +77,21 @@ void OctaveSession::login()
     args << QLatin1String("--eval");
     args << QString::fromLatin1("addpath %1;").arg(octaveScriptInstallDir);
 
+    // Print the temp dir, used for plot files
+    args << QLatin1String("--eval");
+    args << QLatin1String("printf('%s\\n', ['____TMP_DIR____ = ' tempdir]);");
+
+    // Do not show extra text in help commands
+    args << QLatin1String("--eval");
+    args << QLatin1String("suppress_verbose_help_message(1);");
+
     if (OctaveSettings::integratePlots())
     {
         // Do not show the popup when plotting, rather only print to a file
         args << QLatin1String("--eval");
-        args << QLatin1String("graphics_toolkit gnuplot;");
-        args << QLatin1String("--eval");
         args << QLatin1String("set (0, \"defaultfigurevisible\",\"off\");");
+        args << QLatin1String("--eval");
+        args << QLatin1String("graphics_toolkit gnuplot;");
     }
     else
     {
@@ -92,14 +99,6 @@ void OctaveSession::login()
         args << QLatin1String("set (0, \"defaultfigurevisible\",\"on\");");
     }
 
-    // Do not show extra text in help commands
-    args << QLatin1String("--eval");
-    args << QLatin1String("suppress_verbose_help_message(1);");
-
-    // Print the temp dir, used for plot files
-    args << QLatin1String("--eval");
-    args << QLatin1String("____TMP_DIR____ = tempdir");
-
     if (OctaveSettings::integratePlots())
     {
         m_watch = new KDirWatch(this);
@@ -107,23 +106,43 @@ void OctaveSession::login()
         connect (m_watch, SIGNAL(dirty(QString)), SLOT(plotFileChanged(QString)) );
     }
 
-    // connect the signal and slots prior to staring octave to make sure we handle the very first output
-    // in parserOutput() to determine the temp folder and the format of the promt
-    connect ( m_process, SIGNAL (readyReadStandardOutput()), SLOT (readOutput()) );
-    connect ( m_process, SIGNAL (readyReadStandardError()), SLOT (readError()) );
-    connect ( m_process, SIGNAL (error(QProcess::ProcessError)), SLOT (processError()) );
-
     m_process->setProgram ( OctaveSettings::path().toLocalFile(), args );
     qDebug() << "starting " << m_process->program();
     m_process->setOutputChannelMode ( KProcess::SeparateChannels );
     m_process->start();
     m_process->waitForStarted();
-    m_process->waitForReadyRead();
+
+    // Got tmp dir
+    bool loginFinished = false;
+    QString input;
+    while (!loginFinished)
+    {
+        m_process->waitForReadyRead();
+        input += QString::fromLatin1(m_process->readAllStandardOutput());
+        qDebug() << "login input: " << input;
+        if (input.contains(QLatin1String("____TMP_DIR____")))
+            {
+                m_tempDir = input;
+                m_tempDir.remove(0,18);
+                m_tempDir.chop(1); // isolate the tempDir's location
+                qDebug() << "Got temporary file dir:" << m_tempDir;
+                if (m_watch)
+                {
+                    m_watch->addDir(m_tempDir, KDirWatch::WatchFiles);
+                }
+                loginFinished = true;
+            }
+    }
+
+    connect ( m_process, SIGNAL (readyReadStandardOutput()), SLOT (readOutput()) );
+    connect ( m_process, SIGNAL (readyReadStandardError()), SLOT (readError()) );
+    connect ( m_process, SIGNAL (error(QProcess::ProcessError)), SLOT (processError()) );
 
     if(!OctaveSettings::self()->autorunScripts().isEmpty()){
         QString autorunScripts = OctaveSettings::self()->autorunScripts().join(QLatin1String("\n"));
 
         evaluateExpression(autorunScripts, OctaveExpression::DeleteOnFinish, true);
+        updateVariables();
     }
 
     changeStatus(Cantor::Session::Done);
@@ -162,9 +181,8 @@ void OctaveSession::logout()
     m_tempDir.clear();
     m_output.clear();
     m_previousPromptNumber = 1;
-    m_loginFinish = false;
 
-    m_variableModel->clearVariables();
+    variableModel()->clearVariables();
 
     changeStatus(Status::Disable);
 
@@ -185,11 +203,8 @@ void OctaveSession::interrupt()
             ; //TODO: interrupt the process on windows
 #endif
         }
-        expressionQueue().first()->interrupt();
-        expressionQueue().removeFirst();
-
         foreach (Cantor::Expression* expression, expressionQueue())
-            expression->setStatus(Cantor::Expression::Done);
+            expression->setStatus(Cantor::Expression::Interrupted);
         expressionQueue().clear();
 
         // Cleanup inner state and call octave prompt printing
@@ -225,10 +240,7 @@ void OctaveSession::runFirstExpression()
 {
     OctaveExpression* expression = static_cast<OctaveExpression*>(expressionQueue().first());
     connect(expression, SIGNAL(statusChanged(Cantor::Expression::Status)), this, SLOT(currentExpressionStatusChanged(Cantor::Expression::Status)));
-    QString command = expression->command();
-    command.replace(QLatin1String(";\n"), QLatin1String(";"));
-    command.replace(QLatin1Char('\n'), QLatin1Char(','));
-    command += QLatin1Char('\n');
+    QString command = expression->internalCommand();
     expression->setStatus(Cantor::Expression::Computing);
     if (isDoNothingCommand(command))
         expression->setStatus(Cantor::Expression::Done);
@@ -262,32 +274,9 @@ void OctaveSession::readOutput()
     qDebug() << "readOutput";
     while (m_process->bytesAvailable() > 0)
     {
-        if (m_tempDir.isEmpty() && !m_process->canReadLine())
-        {
-            qDebug() << "Waiting";
-            // Wait for the full line containing octave's tempDir
-            return;
-        }
         QString line = QString::fromLocal8Bit(m_process->readLine());
         qDebug()<<"start parsing " << "  " << line;
-        if (!m_loginFinish)
-        {
-            // we're parsing the first output of octave after the start
-            // -> determine the location of the temporary folder
-            if (line.contains(QLatin1String("____TMP_DIR____")))
-            {
-                m_tempDir = line;
-                m_tempDir.remove(0,18);
-                m_tempDir.chop(1); // isolate the tempDir's location
-                qDebug() << "Got temporary file dir:" << m_tempDir;
-                if (m_watch)
-                {
-                    m_watch->addDir(m_tempDir, KDirWatch::WatchFiles);
-                }
-                m_loginFinish = true;
-            }
-        }
-        else if (line.contains(m_prompt))
+        if (line.contains(m_prompt))
         {
             const int promptNumber = m_prompt.cap(1).toInt();
             if (!expressionQueue().isEmpty())
@@ -296,13 +285,7 @@ void OctaveSession::readOutput()
                 if (m_previousPromptNumber + 1 == promptNumber || isSpecialOctaveCommand(command))
                 {
                     if (!expressionQueue().isEmpty())
-                    {
-                        if (command.contains(QLatin1String(" = ")) || command.contains(QLatin1String("clear")))
-                        {
-                            emit variablesChanged();
-                        }
                         static_cast<OctaveExpression*>(expressionQueue().first())->parseOutput(m_output);
-                    }
                 }
                 else
                 {
@@ -334,12 +317,9 @@ void OctaveSession::currentExpressionStatusChanged(Cantor::Expression::Status st
     {
     case Cantor::Expression::Done:
     case Cantor::Expression::Error:
-        expressionQueue().removeFirst();
-        if (expressionQueue().isEmpty())
-            changeStatus(Done);
-        else
-            runFirstExpression();
+        finishFirstExpression();
         break;
+
     default:
         break;
     }
@@ -370,20 +350,9 @@ Cantor::SyntaxHelpObject* OctaveSession::syntaxHelpFor ( const QString& cmd )
 
 QSyntaxHighlighter* OctaveSession::syntaxHighlighter ( QObject* parent )
 {
-    OctaveHighlighter* highlighter = new OctaveHighlighter ( parent, this );
-
-    connect ( this, SIGNAL(variablesChanged()), highlighter, SLOT(updateVariables()) );
-    connect ( this, SIGNAL(statusChanged(Cantor::Session::Status)), highlighter, SLOT(sessionStatusChanged(Cantor::Session::Status)) );
-
-    return highlighter;
+    return new OctaveHighlighter ( parent, this );
 }
 
-QAbstractItemModel* OctaveSession::variableModel()
-{
-    return m_variableModel;
-}
-
-
 void OctaveSession::runSpecificCommands()
 {
     m_process->write("figure(1,'visible','off')");
@@ -391,7 +360,7 @@ void OctaveSession::runSpecificCommands()
 
 bool OctaveSession::isDoNothingCommand(const QString& command)
 {
-    return PROMPT_UNCHANGEABLE_COMMAND.exactMatch(command) || command.isEmpty();
+    return PROMPT_UNCHANGEABLE_COMMAND.exactMatch(command) || command.isEmpty() || command == QLatin1String("\n");
 }
 
 bool OctaveSession::isSpecialOctaveCommand(const QString& command)
