@@ -23,6 +23,7 @@ using namespace Cantor;
 
 #include <KProcess>
 #include <QDebug>
+#include <QDir>
 #include <QFileInfo>
 #include <QEventLoop>
 #include <QTemporaryFile>
@@ -42,6 +43,7 @@ class Cantor::LatexRendererPrivate
     QString errorMessage;
     bool success;
     QString latexFilename;
+    QTemporaryFile* texFile;
 };
 
 static const QLatin1String tex("\\documentclass[12pt,fleqn]{article}"\
@@ -70,6 +72,7 @@ LatexRenderer::LatexRenderer(QObject* parent) : QObject(parent),
     d->isEquationOnly=false;
     d->equationType=InlineEquation;
     d->success=false;
+    d->texFile=nullptr;
 }
 
 LatexRenderer::~LatexRenderer()
@@ -166,8 +169,8 @@ void LatexRenderer::render()
 void LatexRenderer::renderBlocking()
 {
     QEventLoop event;
-    connect(this, SIGNAL(done()), &event, SLOT(quit()));
-    connect(this, SIGNAL(error()), &event, SLOT(quit()));
+    connect(this, &LatexRenderer::done, &event, &QEventLoop::quit);
+    connect(this, &LatexRenderer::error, &event, &QEventLoop::quit);
 
     render();
     event.exec();
@@ -178,9 +181,11 @@ void LatexRenderer::renderWithLatex()
     qDebug()<<"rendering using latex method";
     QString dir=QStandardPaths::writableLocation(QStandardPaths::TempLocation);
 
-    //Check if the cantor subdir exists, if not, create it
-    QTemporaryFile *texFile=new QTemporaryFile(dir + QLatin1String("/cantor_tex-XXXXXX.tex"));
-    texFile->open();
+    if (d->texFile)
+        delete d->texFile;
+
+    d->texFile=new QTemporaryFile(dir + QDir::separator() + QLatin1String("cantor_tex-XXXXXX.tex"));
+    d->texFile->open();
 
     KColorScheme scheme(QPalette::Active);
     const QColor &backgroundColor=scheme.background().color();
@@ -201,17 +206,17 @@ void LatexRenderer::renderWithLatex()
 
 //     qDebug()<<"full tex:\n"<<expressionTex;
 
-    texFile->write(expressionTex.toUtf8());
-    texFile->flush();
+    d->texFile->write(expressionTex.toUtf8());
+    d->texFile->flush();
 
-    QString fileName = texFile->fileName();
+    QString fileName = d->texFile->fileName();
     qDebug()<<"fileName: "<<fileName;
     d->latexFilename=fileName;
     d->latexFilename.replace(QLatin1String(".tex"), QLatin1String(".eps"));
     KProcess *p=new KProcess( this );
     p->setWorkingDirectory(dir);
 
-    (*p)<<Settings::self()->latexCommand()<<QLatin1String("-interaction=batchmode")<<QLatin1String("-halt-on-error")<<fileName;
+    (*p)<<Settings::self()->latexCommand()<<QStringLiteral("-interaction=batchmode")<<QStringLiteral("-halt-on-error")<<fileName;
 
     connect(p, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(convertToPs()) );
     p->start();
@@ -219,12 +224,11 @@ void LatexRenderer::renderWithLatex()
 
 void LatexRenderer::convertToPs()
 {
-    qDebug()<<"converting to ps";
     QString dviFile=d->latexFilename;
     dviFile.replace(QLatin1String(".eps"), QLatin1String(".dvi"));
     KProcess *p=new KProcess( this );
-    qDebug()<<"running: "<<Settings::self()->dvipsCommand()<<"-E"<<"-o"<<d->latexFilename<<dviFile;
-    (*p)<<Settings::self()->dvipsCommand()<<QLatin1String("-E")<<QLatin1String("-o")<<d->latexFilename<<dviFile;
+    qDebug()<<"converting to eps: "<<Settings::self()->dvipsCommand()<<"-E"<<"-o"<<d->latexFilename<<dviFile;
+    (*p)<<Settings::self()->dvipsCommand()<<QStringLiteral("-E")<<QStringLiteral("-o")<<d->latexFilename<<dviFile;
 
     connect(p, SIGNAL(finished(int,QProcess::ExitStatus)), this, SLOT(convertingDone()) );
     p->start();
@@ -232,27 +236,26 @@ void LatexRenderer::convertToPs()
 
 void LatexRenderer::convertingDone()
 {
-    qDebug()<<"rendered file "<<d->latexFilename;
-    //cleanup the temp directory a bit...
-    QString dir=QStandardPaths::writableLocation(QStandardPaths::TempLocation) + QLatin1String("/") + QLatin1String("cantor");
-    QStringList unneededExtensions;
-    unneededExtensions<<QLatin1String(".log")<<QLatin1String(".aux")<<QLatin1String(".tex")<<QLatin1String(".dvi");
-    foreach(const QString& ext, unneededExtensions)
-    {
-        QString s=d->latexFilename;
-        s.replace(QLatin1String(".eps"), ext);
-        QFile f(s);
-        //f.remove();
-    }
+    QFileInfo info(d->latexFilename);
+    qDebug() <<"remove temporary files for " << d->latexFilename;
 
-    if(QFileInfo(d->latexFilename).exists())
+    delete d->texFile;
+    d->texFile = nullptr;
+
+    QString pathWithoutExtention = info.path() + QDir::separator() + info.completeBaseName();
+    QFile::remove(pathWithoutExtention + QLatin1String(".log"));
+    QFile::remove(pathWithoutExtention + QLatin1String(".aux"));
+    QFile::remove(pathWithoutExtention + QLatin1String(".tex"));
+    QFile::remove(pathWithoutExtention + QLatin1String(".dvi"));
+
+    if(info.exists())
     {
         d->success=true;
         emit done();
     }else
     {
         d->success=false;
-        setErrorMessage(QLatin1String("something is wrong"));
+        setErrorMessage(QStringLiteral("failed to create the latex preview image"));
         emit error();
     }
 }
@@ -262,5 +265,3 @@ void LatexRenderer::renderWithMml()
     qDebug()<<"WARNING: MML rendering not implemented yet!";
     emit done();
 }
-
-
