@@ -22,6 +22,7 @@
 using namespace Cantor;
 
 #include "backend.h"
+#include "defaultvariablemodel.h"
 
 #include <QTimer>
 #include <QQueue>
@@ -29,7 +30,7 @@ using namespace Cantor;
 class Cantor::SessionPrivate
 {
   public:
-    SessionPrivate() : backend(nullptr), status(Session::Disable), typesettingEnabled(false), expressionCount(0)
+    SessionPrivate() : backend(nullptr), status(Session::Disable), typesettingEnabled(false), expressionCount(0), variableModel(nullptr), needUpdate(false)
     {
     }
 
@@ -38,6 +39,8 @@ class Cantor::SessionPrivate
     bool typesettingEnabled;
     int expressionCount;
     QList<Cantor::Expression*> expressionQueue;
+    DefaultVariableModel* variableModel;
+    bool needUpdate;
 };
 
 Session::Session( Backend* backend ) : QObject(backend), d(new SessionPrivate)
@@ -45,6 +48,12 @@ Session::Session( Backend* backend ) : QObject(backend), d(new SessionPrivate)
     d->backend=backend;
 }
 
+Session::Session( Backend* backend, DefaultVariableModel* model) : QObject(backend), d(new SessionPrivate)
+{
+    d->backend=backend;
+    d->variableModel=model;
+}
+
 Session::~Session()
 {
     delete d;
@@ -62,8 +71,8 @@ void Session::enqueueExpression(Expression* expr)
     //run the newly added expression immediately if it's the only one in the queue
     if (d->expressionQueue.size() == 1)
     {
-     changeStatus(Cantor::Session::Running);
-     runFirstExpression();
+        changeStatus(Cantor::Session::Running);
+        runFirstExpression();
     }
     else
         expr->setStatus(Cantor::Expression::Queued);
@@ -74,6 +83,29 @@ void Session::runFirstExpression()
 
 }
 
+void Session::finishFirstExpression(bool setDoneAfterUpdate)
+{
+    if (!d->expressionQueue.isEmpty())
+        d->needUpdate |= !d->expressionQueue.takeFirst()->isInternal();
+
+    if (d->expressionQueue.isEmpty())
+        if (d->variableModel && d->needUpdate)
+        {
+            d->variableModel->update();
+            d->needUpdate = false;
+
+            // Some variable models could update internal lists without running expressions
+            // So, if after update queue still empty, set status to Done
+            // setDoneAfterUpdate used for compatibility with some backends, like R
+            if (setDoneAfterUpdate && d->expressionQueue.isEmpty())
+                changeStatus(Done);
+        }
+        else
+            changeStatus(Done);
+    else
+        runFirstExpression();
+}
+
 Backend* Session::backend()
 {
     return d->backend;
@@ -133,12 +165,32 @@ QSyntaxHighlighter* Session::syntaxHighlighter(QObject* parent)
     return nullptr;
 }
 
-QAbstractItemModel* Session::variableModel()
+DefaultVariableModel* Session::variableModel() const
 {
-    //Return 0 per default, so Backends not offering variable management don't
+    //Return deafult session model per default
+    //By default, variableModel is nullptr, so Backends not offering variable management don't
     //have to reimplement this. This method should only be called on backends with
     //VariableManagement Capability flag
-    return nullptr;
+    return d->variableModel;
+}
+
+QAbstractItemModel* Session::variableDataModel() const
+{
+    return variableModel();
+}
+
+void Session::updateVariables()
+{
+    if (d->variableModel)
+    {
+        d->variableModel->update();
+        d->needUpdate = false;
+    }
+}
+
+void Cantor::Session::setVariableModel(Cantor::DefaultVariableModel* model)
+{
+    d->variableModel = model;
 }
 
 int Session::nextExpressionId()
